#!/usr/bin/python
#
#    MidiIn2.py -- a higher-level interface to MidiIn
#
#    Copyright (C) 1999  Eric S. Tiedemann
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Library General Public
#    License as published by the Free Software Foundation; either
#    version 2 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this library; if not, write to the Free
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Eric S. Tiedemann, est@hyperreal.org
#

# should check that callbacks are callable
# defaulting/uncurrying assignment
# all-channels mode
# need to make complete re number of args etc

import sys
import MidiIn
from types import *

class MidiIn2:
    class CB:
        def __init__(self):
            nons = [None] * 17
            for c in range(0, 17):
                nons[c] = [None] * 128

            noffs = [None] * 17
            for c in range(0, 17):
                noffs[c] = [None] * 128

            ccs = [None] * 17
            for c in range(0, 17):
                ccs[c] = [None] * 128

            self.tab = {'note-on':nons,
                        'note-off':noffs,
                        'control-change':ccs,
                        }

        def run(self, *args):
            #print args
            cmd = self.tab.get(args[0])
            if (cmd):
                if type(cmd) is ListType:
                    chcmd = cmd[args[1]]
                    if type(chcmd) is ListType:
                        cmd = chcmd[args[2]]
                        if cmd: apply(cmd, tuple(args[3:]))
                    elif chcmd:
                        apply(chcmd, tuple(args[2:]))
                else:
                    apply(cmd, ())

        # this is returned for indexing into channels
        class chidx:
            # this is returned for indexing below channels
            class subchidx:
                def __init__(self, subchs):
                    self.subchs = subchs
                def __getitem__(self, i):
                    if type(i) != IntType:
                        raise TypeError, \
                              "sub-channel index isn't an integer: %s" % str(i)
                    # this should be different for controllers
                    if (i >= 0 < 128):
                        return self.subchs[i]
                    else:
                        raise IndexError, \
                              "sub-channel index %d is out of range" % i
                def __setitem__(self, i, v):
                    if type(i) != IntType:
                        raise TypeError, \
                              "sub-channel index isn't an integer: %s" % str(i)
                    # this should be different for controllers
                    if (i >= 0 < 128):
                        self.subchs[i] = v
                    else:
                        raise IndexError, \
                              "sub-channel index %d is out of range" % i
                    
            def __init__(self, chs):
                self.chs = chs
            def __getitem__(self, i):
                if type(i) != IntType:
                    raise TypeError, \
                          "channel index isn't an integer: %s" % str(i)
                if (i > 0 < 17):
                    v = self.chs[i]
                    if type(v) is ListType:
                        return self.subchidx(v)
                    else:
                        return v
                else:
                    raise IndexError, "channel index %d is out of range" % i
            
        def __getitem__(self, a):
            if self.tab.has_key(a):
                v = self.tab[a]
                if type(v) is ListType:
                    return self.chidx(v)
                else:
                    return v
            else:
                raise AttributeError, "unhandled message type: %s" % str(a)
        
            # needs to return an appropriate trampoline
            # when we reach a terminal level of getting, we just
            # return a value..other wise we return an object that
            # can respond to get and set ops
            print 99
            pass

    def __init__(self):
        self.cb0 = self.CB()
        self.m = MidiIn.MidiIn(self.cb0.run)

    def inbytes(self, bs):
        self.m.inbytes(bs)
        
    def __setattr__(self, a, v):
        if (a == 'cb'):
            self.cb0.set_global(v)
        else:
            self.__dict__[a] = v

    def __getattr__(self, a):
        if (a == 'cb'):
            return self.cb0
        else:
            return self.__dict__[a]

if (__name__ == '__main__'):
    import os
    import fcntl
    import FCNTL
    import select
    import time
    def baz(n, arg):
        print n, arg
    def foo(n):
        return lambda arg, n=n: baz(n, arg)
    m = MidiIn2()
    for i in range(1, 65):
        m.cb['control-change'][5][i] = foo(i)
    fcntl.fcntl(0, FCNTL.F_SETFL,
                (os.O_NONBLOCK | fcntl.fcntl(0, FCNTL.F_GETFL)))
    n = 0
    while (n < 10000):
        select.select([0], [], [])
        n = n + 1
        c = os.read(0, 1000)
        print len(c)
        m.inbytes(c)
        time.sleep(.01)
